;;; muac.lsp 
;;; Some functions for use with "muac" from AutoCad
;;;
;;; usage: from the command line of AutoCad: (load "muac.lsp")
;;;
;;; Once loaded, the following commands are defined:
;;;
;;; The MUAC command writes a file to be read by muac, containing the lists of nodes and bars
;;; resulting of the selected drawing entities, that must be lines. The cross-section code 
;;; of each bar is set as its line color code (bylayer=1, red=1, yellow=2, etc).
;;; The remaining data (sections, materials, loads, displacement conditions) must be entered by hand.
;;;
;;; The MUAC-READ command reads a muac output file, and generates internal data structures
;;; with these results.     
;;;
;;; Once you have read a muac output file with the MUAC-READ command, the following 
;;; commands are available:
;;;
;;; NUMERATE: draws the node numbers
;;;
;;; EXERTIONS: draws the internal forces of the bars by using a color code:
;;; from lowest (red) to highest (blue) values. Tensions are drawn with a continuous line, 
;;; compressions are drawn with dashed lines.
;;;
;;; QUERY-BAR: After pointing a bar in the screen, indicates the value 
;;; of the internal force of the designated bar.  
   



(setq PEQ 0.0001)
(setq CASIPEQ 0.005)
(setq MUY-GRANDE 1e100)
(setq GRAD-RAD (/ 180 3.141592))
(setq DEPURAR nil)


;; funciones auxiliares generales
(defun first (lista) (nth 0 lista)) 
(defun second (lista) (nth 1 lista))
(defun third (lista) (nth 2 lista))
(defun cuadrado (num) (* num num))

(defun deb (imprimir)
  (if DEPURAR
      (progn
	(print "**** --->")
	(princ  imprimir))))

(defun redond (num dec)
  ;; (if (not (and (integerp dec) (>= dec 0)))
  ;;     (error "n-decimales ha de ser entero >= 0"))
  (atof (rtos num 2 dec))) ; 2 es modo decimal

(defun numero-par (num)
  (if (equal (rem num 2) 0)
      T
      nil))

(defun numero-mult3 (num)
  (if (equal (rem num 3) 0)
      T
      nil))

(defun integerp (a)
  (if (= (type a) 'INT)
      T
      nil))

(defun quita-nil (lista / lista-tmp) ; quita elementos nil de una lista
  (if (and lista (car lista))
      (setq lista-tmp (cons (car lista)(quita-nil (cdr lista))))
      (if lista
	  (quita-nil (cdr lista))
	  lista-tmp)))
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
		 

(setq FACTOR-TEXTO 0.01)
(setq PRECISION 5)

(setq LISTA-NODOS nil)
(setq LISTA-LINEAS nil)
(setq LISTA-ESFUERZOS nil)
(setq LISTA-DESPLAZAMIENTOS nil)
(setq LISTA-BIELAS nil)

(defun lista-nodos-int (/ sel-set lista-nombres 
			n-lineas linea nombre
			punto posicion
			node-number pt secc)
  (setq
   sel-set NIL
   lista-nombres NIL
   LISTA-LINEAS NIL
   n-lineas NIL
   linea NIL
   nombre NIL
   LISTA-NODOS nil
   punto NIL posicion NIL
   node-number NIL)
  (setq sel-set (ssget '((0 . "LINE"))))
  (setq lista-nombres (reverse (set-to-list sel-set)))
  (foreach nombre lista-nombres
	   (setq linea
		  (list 
		   (cdr (assoc 10 (entget nombre)))
		   (cdr (assoc 11 (entget nombre)))))
	   (foreach pt linea
		    (if (setq posicion
			      (vl-position
			       (mapcar '(lambda(x) (redond x PRECISION))  pt)
			       LISTA-NODOS))
			(setq linea (subst (1+ posicion) pt linea))
			(progn
			  (setq LISTA-NODOS (append
					     LISTA-NODOS
					     (list (mapcar '(lambda(x) (redond x PRECISION)) pt))))
			  (setq linea (subst (length LISTA-NODOS) pt linea)))))
	   (setq linea (append linea
			       (list (if (setq secc (cdr (assoc 62 (entget nombre))))
					 secc
					 1)))) ; el tipo de sección es el color de la línea
	                                       ; y si es bylayer o byblock -> 1  
	   (setq LISTA-LINEAS (cons linea LISTA-LINEAS)))
  (setq LISTA-LINEAS (reverse LISTA-LINEAS))
  )


(defun set-to-list (sel-set / n-set i lista)
  (setq n-set (sslength sel-set) i 0 lista NIL)
  (while (< i n-set)
    (setq lista (cons (ssname sel-set i) lista))
    (setq i (1+ i)))
  lista)
 
(defun rotula-nodos (/ nodo xmin ymin xmax ymax x y h l textalt i)
(setq xmin MUY-GRANDE ymin MUY-GRANDE xmax (- MUY-GRANDE) ymax (- MUY-GRANDE)
	x NIL y NIL h NIL l NIL textalt nil i 0)
  (foreach nodo LISTA-NODOS
	   (progn
	     (if (< (setq x (first nodo)) xmin) (setq xmin x))
	     (if (< (setq y (second nodo)) ymin) (setq ymin y))
	     (if (> x xmax) (setq xmax x))
	     (if (> y ymax) (setq ymax y))))
  (setq h (abs (- ymax ymin)))
  (setq l (abs (- xmax xmin)))
  (setq textalt (* (max h l) FACTOR-TEXTO))
  (foreach nodo LISTA-NODOS
	   (progn
	     (command "_TEXT" nodo textalt 0 (setq i (1+ i)))))
  )

(defun C:NUMERATE () (rotula-nodos))
(defun C:MUAC (/ fd nodo barra i secc)
  (setq i 0)
  (lista-nodos-int)
  (rotula-nodos)
  (setq fd (open (getstring "    Nombre del archivo?: ") "w"))
  (write-line "C   #coordenadas x y de los nodos" fd)
  (foreach nodo LISTA-NODOS
	   (write-line (strcat (rtos (first nodo))
			       " "
			       (rtos (second nodo))
 			       "  #"  (itoa (setq i (1+ i))))
		       fd))
  (write-line "B   #barras nodo-in nodo-fin secc-type[que se corresponde con el color]" fd)
  (setq i 0)
  (foreach barra LISTA-LINEAS
	   (write-line
	    (strcat (itoa (first barra))
		    " "
		    (itoa (second barra))
		    " "
		    (if (and (setq secc (third barra))
			     (not (zerop secc)))
			(itoa secc) ; el tipo de barra es el color de la línea!!
			"1")        ; si es bylayer o byblock o color 0 ---> sección 1
		                    ; ya es siempre un número por lo cual es redundante (no puede ser nil)
		    "  #"  (itoa (setq i (1+ i))))
		    fd))
  (close fd)
  (prin1)
  )

;; PARA LEER FICHEROS DE SALIDA DE MUAC

(setq PALABRA-RES '(B C D d E e H h N M Q R r S V X)
      COMENTARIO "#")

(defun mi-atof (str)
  (if (equal 0 (strlen str))
      nil
      (if (or
	   (vl-string-position (ascii ".") str) 
	   (vl-string-position (ascii "E") str) 
	   (vl-string-position (ascii "e") str)) 
	  (atof str)
	  (atoi str))))

(defun quita-comentario-y-tab (texto / pos)
  (while (vl-string-search "\t" (setq texto
				      (vl-string-subst " " "\t" texto))))
  (setq pos (vl-string-position (ascii COMENTARIO) texto))
  (if pos
      (setq texto (substr texto 1 pos)))
  texto)

(defun quita-letra-inicio-lista (texto)
  (vl-string-trim " " (substr texto 3)))

(defun es-inicio-de-lista (texto / letra)
    (if (member (setq letra (read texto)) PALABRA-RES)
	letra
	nil))

(defun lee-linea (file / lin)
  (while (equal "" (setq lin (read-line file)))) ; saltar líneas en blanco
  (if lin
      (vl-string-trim
       " "
       (quita-comentario-y-tab lin))
      nil))

(defun lee-linea-numeros (texto / pos num lista-num)
  (setq lista-num nil)
  (while (setq num (mi-atof (setq texto (vl-string-trim " " texto))))
    (setq lista-num (cons num lista-num))
    (setq pos (vl-string-position (ascii " ") texto))
    (if pos
	(setq texto (substr texto (+ 2 pos)))
	(setq texto "")))
  (reverse lista-num))

(defun linea-numeros-to-puntos (lista-num / num punto lista-puntos i)
  (setq i 0 lista-puntos nil)
  (foreach num lista-num
	   ;; (if (integerp num)
	   ;;     (setq num (float num))) ;; PARA ESF-BAR
	   (setq num (redond num PRECISION)) ;; PARA ESF-BAR
	   (if (numero-par (setq i (1+ i)))
	       (setq lista-puntos (append lista-puntos
					  (list (append punto (list num  0.0)))))
	       (setq punto (list num))))
  lista-puntos)

(defun linea-numeros-to-barras (lista-num / num barra lista-barras i)
  (setq i 0 lista-barras nil)
  (foreach num lista-num
	   (if (numero-mult3 (setq i (1+ i)))
	       (setq i 0 lista-barras (append lista-barras (list barra)))
	       (if (numero-par i)
		   (setq barra (append barra (list num)))
		   (setq barra (list num)))))
  lista-barras)

(defun linea-numeros-to-desp (lista-num / num desp lista-desps i)
  (setq i 0 lista-desps nil)
  (foreach num lista-num
	   (if (numero-par (setq i (1+ i)))
	       (setq lista-desps (append lista-desps
					  (list (append desp (list num )))))
	       (setq desp (list num))))
  lista-desps)


    

(defun C:MUAC-READ ( / fd texto letra)
  (setq LISTA-NODOS nil
	LISTA-LINEAS nil
	LISTA-ESFUERZOS nil
	LISTA-DESPLAZAMIENTOS nil)
  (setq fd (open (getstring "¿Nombre del archivo a leer?: ") "r"))
  (setq texto (lee-linea fd))
  (while texto
    (if (setq letra (es-inicio-de-lista texto))
	(progn
	  (if (equal letra 'C) ;; nodos
	      (progn
		(setq texto (quita-letra-inicio-lista texto))
		(while (and texto (not (es-inicio-de-lista texto)))
		  (setq LISTA-NODOS (append
					LISTA-NODOS
					(linea-numeros-to-puntos (lee-linea-numeros texto))))
		  (setq texto (lee-linea fd)))))
	  (if (equal letra 'B) ;; nodos
	      (progn
		(setq texto (quita-letra-inicio-lista texto))
		(while (and texto (not (es-inicio-de-lista texto)))
		  (setq LISTA-LINEAS (append
				      LISTA-LINEAS
				      (linea-numeros-to-barras (lee-linea-numeros texto))))
		  (setq texto (lee-linea fd)))))
	  (if (or ;; saltar el resto de datos de entrada
	       (equal letra 'M)  
	       (equal letra 'S)  
	       (equal letra 'Q)  
	       (equal letra 'V))  
	      (progn
		(setq texto (quita-letra-inicio-lista texto))
		(while (and texto (not (es-inicio-de-lista texto)))
		  (setq texto (lee-linea fd)))))
	  (if (or ;; esfuerzos en las barras
	       (equal letra 'H)  
	       (equal letra 'h)  
	       (equal letra 'N))  
	      (progn
		(setq texto (quita-letra-inicio-lista texto))
		(while (and texto (not (es-inicio-de-lista texto)))
		  (setq LISTA-ESFUERZOS (append
					 LISTA-ESFUERZOS
					 (lee-linea-numeros texto)))
		  (setq texto (lee-linea fd)))))
	  (if (or ;; desplazamientos en los nodos
	       (equal letra 'D)  
	       (equal letra 'd))  
	      (progn
		(setq texto (quita-letra-inicio-lista texto))
		(while (and texto (not (es-inicio-de-lista texto)))
		  (setq LISTA-DESPLAZAMIENTOS (append
					 LISTA-DESPLAZAMIENTOS
					 (linea-numeros-to-desp (lee-linea-numeros texto))))
		  (setq texto (lee-linea fd)))))
	  (if (or ;; por ahora saltamos algunos resultados
	       (equal letra 'e)  
	       (equal letra 'E)  
	       (equal letra 'r)  
	       (equal letra 'R))  
	      (progn
		(setq texto (quita-letra-inicio-lista texto))
		(while (and texto (not (es-inicio-de-lista texto)))
		  (setq texto (lee-linea fd))))))))
  (close fd))
 

(setq COLORES-ESFUERZOS '(20 30 40 50 60 70 80 90 100 110 120 130 140 150 160 170))
 
(defun C:EXERTIONS ( / emin emax esf  i col escala-col)
  (command "_-LAYER" "_MAKE" "muac_esf" "")
   (if (not (tblsearch "LTYPE" "ACAD_ISO02W100"))
       (command "_-LINETYPE" "_Load" "ACAD_ISO02W100" "acadiso.lin" ""))
  (command "_LTSCALE" "0.0025")
	   
  (setq emin MUY-GRANDE   emax (- MUY-GRANDE) esf 0.0)
  (foreach esf LISTA-ESFUERZOS
  	     (if (<  esf  emin) (setq emin esf))
  	     (if (>  esf  emax) (setq emax esf)))
  (setq escala-col (/ (- emax emin) (1- (length COLORES-ESFUERZOS))))
  (setq i -1)
  (foreach barra LISTA-LINEAS
  	   (setq i (1+ i))
  	   (setq esf (nth i LISTA-ESFUERZOS))
  	   (setq col (fix (redond (/ (- esf emin) escala-col) 0)))
  	   (command "_-COLOR" (itoa (nth col COLORES-ESFUERZOS)))
	   (command "_-LINETYPE" "_s"
		    (if (>= esf 0.0)
			"_bylayer"
			"ACAD_ISO02W100") 
		    "") 
  	   (command "_LINE"
  		    (nth (1- (first barra)) LISTA-NODOS)  
  		    (nth (1- (second barra)) LISTA-NODOS) "")))  
  

(defun C:QUERY-BAR (/ barra nbarra linea posicion)
  (setq barra nil nbarra nil linea nil posicion nil numbarra nil)
  (setq nbarra (first (entsel "Designar barra ")))
  (setq linea (list 
	       (cdr (assoc 10 (entget nbarra)))
	       (cdr (assoc 11 (entget nbarra)))))
  (foreach pt linea
	   (setq posicion
		 (vl-position
		  (mapcar '(lambda(x) (redond x PRECISION))  pt)
		  LISTA-NODOS))
	   (setq barra (cons (1+ posicion) barra)))
  (setq numbarra (if (setq posicion (vl-position barra LISTA-LINEAS))
		     (1+ posicion)
		     (1+ (vl-position (reverse barra) LISTA-LINEAS))))
  (write-line (strcat
	       "BARRA "
	       (itoa numbarra) " N = "
	       (rtos (nth (1-  numbarra) LISTA-ESFUERZOS))))
  nil)
